/*
 * File: Action.cpp
 *
 * Description: Action class implementation
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Action.h"
#include "ParseTree.h"
#include "AgentBodyStates.h"
#include "LogLevels.h"
using namespace std;

Action::Action() :
    haveStarter(false), runType(CONTINUOUS), activeSection(0), activeStateNo(0),
            stop(false), locked(false)
{
}

Action::~Action()
{
}

void Action::LoadAction(ParseTree *action, AgentBodyStates &agentStates)
{
    for (unsigned i = 0; i < action->GetNumOfKeys(); ++i)
    {
        if (action->GetKey(i) == "Init")
        {
            Log(LOG_ACTIONPARSER) << "Loading Init states...";
            LoadStates(0, action->GetData(i), agentStates);
        }
        else if (action->GetKey(i) == "Acting")
        {
            Log(LOG_ACTIONPARSER) << "Loading Acting states...";
            LoadStates(1, action->GetData(i), agentStates);
        }
        else if (action->GetKey(i) == "Finish")
        {
            Log(LOG_ACTIONPARSER) << "Loading Finish states...";
            LoadStates(2, action->GetData(i), agentStates);
        }
    }
    if (actStates[0].empty())
        activeSection = 1;
}

void Action::LoadStates(int statesType, ParseTree *parser, AgentBodyStates &agentStates)
{
    for (unsigned i = 0; i < parser->GetNumOfKeys(); ++i)
    {
        if (parser->GetKey(i) == "Type")
        {
            if (parser->GetData(i)->GetKey(0) == "runonce")
            {
                runType = RUNONCE;
                Log(LOG_ACTIONPARSER) << "\tAction type is RUNONCE";
            }
            else
            {
                runType = CONTINUOUS;
                Log(LOG_ACTIONPARSER) << "\tAction type is CONTINUOUS";
            }
        }
        else if (parser->GetKey(i) == "state")
        {
            ParseTree *stateParser = parser->GetData(i);
            State tmpState;
            for (unsigned i = 0; i < stateParser->GetNumOfKeys(); ++i)
            {
                if (stateParser->GetKey(i) == "name")
                {
                    string stname = stateParser->GetData(i)->GetKey(0);
                    tmpState.state = agentStates.GetStateNumber(stname);
                    Log(LOG_ACTIONPARSER) << "\tAdd state: " << stname;
                }
                else if (stateParser->GetKey(i) == "starter")
                {
                    if (stateParser->GetData(i)->GetKey(0) == "true")
                        tmpState.starter = true;
                    Log(LOG_ACTIONPARSER) << "\tState is starter";
                }
            }
            if (!haveStarter && !tmpState.starter)
                tmpState.starter = true;
            haveStarter = true;
            actStates[statesType].push_back(tmpState);
        }
    }
}

int Action::NextState() const
{
    if (activeStateNo + 1 < actStates[activeSection].size())
        return actStates[activeSection][activeStateNo + 1].state;

    if (activeSection == 0)
        return actStates[1][0].state;
    else if (activeSection == 1)
    {
        if (runType == RUNONCE || stop == true)
        {
            if (actStates[2].empty())
                return AgentBodyStates::ILLEGAL_STATE;
            return actStates[2][0].state;
        }
        else
            return actStates[1][0].state;
    }

    return AgentBodyStates::ILLEGAL_STATE;
}

bool Action::ChangeState()
{
    if (locked)
    {
        locked = false;
        return true;
    }

    if (activeStateNo + 1 < actStates[activeSection].size())
        activeStateNo++;
    else if (activeSection == 0)
    {
        activeSection = 1;
        activeStateNo = 0;
    }
    else if (activeSection == 1)
    {
        if (runType == RUNONCE || stop == true)
        {
            if (actStates[2].empty())
                return false;
            activeSection = 2;
            activeStateNo = 0;
        }
        else
            activeStateNo = 0;
    }
    else
        return false;

    return true;
}
