/*
 * File: ActionManager.h
 *
 * Description: Action Manager!
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef ACTIONMANAGER_H_
#define ACTIONMANAGER_H_

#include <vector>
#include <map>
#include "Action.h"
#include "AgentBodyStates.h"
#include "TransitionManager.h"

class WorldModel;

class ActionManager
{
    public:
        static const int ILLEGAL_ACT = -1;

        ActionManager(WorldModel &wm);
        virtual ~ActionManager();

        void LoadFile(const char *filename);
        void SetActionType( int acttype, const char *name );
        int ActionType(const char *actName) const;
        void GotoNextStep();

        void UpdateParameters()
        {
            currentTransition->UpdateParameters();
        }

        TransitionData::JointTransitionVector &JointActions()
        {
            return currentTransition->JointActions();
        }

        TransitionData &CurrentTransition()
        {
            return *currentTransition;
        }

        void SetNextAction( int act )
        {
            if (currentActType == ILLEGAL_ACT)
                StartActing(act);
            else if (act == currentActType)
                currentAction->Stop(false);
            else if (act != currentActType)
            {
                currentAction->Stop();
                nextActType = act;
            }
        }

        void SetTransitionParameter(const char *sourceStateName,
                                    const char *destStateName, const char *name, float val)
        {
            transitions.SetTransitionParameter(
                                                agentStates.GetStateNumber(sourceStateName),
                                                agentStates.GetStateNumber(destStateName),
                                                name, val);
        }

        bool IsActing() const
        {
            return currentActType != ILLEGAL_ACT;
        }

        float StepStartTime() const
        {
            return stepStartTime;
        }

        void Stop()
        {
            if (currentAction)
                currentAction->Stop();
        }

        int GetLastState() const
        {
            return prevState;
        }

        bool isStateChanged() const
        {
            return stateChanged;
        }

        int GetStateNumber( const char *name ) const
        {
            return agentStates.GetStateNumber(name);
        }

    private:
        typedef std::vector<Action> ActionVector;

        ActionVector actions;
        std::vector<int> actionIDs;
        std::map<std::string, int> actionNumberMap;
        AgentBodyStates agentStates;
        TransitionManager transitions;
        int currentActType;
        int nextActType;
        float stepStartTime;
        WorldModel &world;
        int prevState;
        int nextState;
        Action *currentAction;
        TransitionData *currentTransition;
        bool stateChanged;

        void StartActing(int act);
};

#endif /*ACTIONMANAGER_H_*/
