/*
 * File: AgentBodyStates.h
 *
 * Description: It stores the agent's state set
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef AGENTBODYSTATES_H_
#define AGENTBODYSTATES_H_

#include <vector>
#include <map>
#include <string>
#include "AgentBody.h"
#include "Exception.h"

class ParseTree;

class AgentBodyStates
{
    public:
        static const int ILLEGAL_STATE = -1;

        AgentBodyStates();
        virtual ~AgentBodyStates();

        void LoadState(ParseTree *stateTree);
        int GetStateNumber( const string &stateName ) const;

        std::string StateName( int state ) const
        {
#ifdef NDEBUG
            return stateNames[state];
#else
            return stateNames.at(state);
#endif
        }

    private:
        typedef std::map<std::string, int> StateNameMap;

        std::vector<AgentBody> states;
        StateNameMap stateNo;
        std::vector<std::string> stateNames;
};

inline int AgentBodyStates::GetStateNumber( const string &stateName ) const
{
    StateNameMap::const_iterator noptr = stateNo.find(stateName);
    ThrowIf(noptr==stateNo.end(), string("State not found: ") + stateName);
    return noptr->second;
}

#endif /*AGENTBODYSTATES_H_*/
