/*
 * File: MessageComposer.h
 *
 * Description: MessageComposer class used in logging
 *
 * Author: UI-AI 3D Robocup Soccer Team <ui_3dsoccer@eng.ui.ac.ir>, (c) 2007
 *
 */

/*
    Copyright (C) 2009  UI-AI 3D Team

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MESSAGECOMPOSER_H_
#define MESSAGECOMPOSER_H_

#include <iostream>
#include <sstream>
#include <boost/shared_ptr.hpp>

class MessageComposer
{
    public:
        explicit MessageComposer(bool active, std::ostream *output) :
            active(active), messageStream(new std::stringstream), output(output)
        {
        }

        template<typename Type>
        MessageComposer &operator<<(Type var)
        {
            if (active)
                (*messageStream) << var;
            return *this;
        }

        ~MessageComposer()
        {
            if (active && messageStream.unique())
                (*output) << messageStream->str() << '\n';
        }

    protected:
        const bool active;
        boost::shared_ptr<std::stringstream> messageStream;
        std::ostream *output;
};

#endif /*MESSAGECOMPOSER_H_*/
